@@ -1,12 +1,13 @@
 
 # -*-Fundamental-*- 
-require 5;    # Time-stamp: "2000-12-09 12:30:56 MST"
+require 5;    # Time-stamp: "2001-03-18 00:36:10 MST"
 package Sort::ArbBiLex;
 use strict;
 use vars qw(@ISA $Debug $VERSION);
-$VERSION = 3.32;
+$VERSION = 3.40;
 $Debug = 0;
 use Carp;
+use integer; # vroom vroom
 
 #POD at end 
 ###########################################################################
@@ -273,6 +274,58 @@ EOVOODOO2
   return $code;
 }
 
+###########################################################################
+
+# "cmp" returns -1, 0, or 1 depending on whether the left argument is
+# stringwise less than, equal to, or greater than the right argument.
+
+sub xcmp {
+  carp "usage: xcmp(\\&sorter,$a,$b)" unless @_ and ref($_[0]);
+  return  0 if $_[1] eq $_[2];  # We have to trap this early.
+  return  1 if $_[1] ne ( $_[0]->($_[1], $_[2]) )[0];
+   # If they were switched when sorted, then the original-first was
+   #  lexically GT than the original-second.
+  return -1 if $_[1] eq ( $_[0]->($_[2], $_[1]) )[0];
+   # If they were switched BACK when REVERSED and sorted, then the
+   #  original-first was lexically LT than the original-second.
+  return  0;
+   # Otherwise they were lexically identical.
+}
+
+# And two actually simpler ones:
+
+sub xlt {
+  carp "usage: xlt(\\&sorter,$a,$b)" unless @_ and ref($_[0]);
+  #AKA: xcmp(@_) == -1;
+  return  0 if $_[1] eq $_[2];  # We have to trap this early.
+  return  1 if $_[1] eq ( $_[0]->($_[2], $_[1]) )[0];
+   # If they were switched BACK when REVERSED and sorted, then the
+   #  original-first was lexically LT than the original-second.
+  return  0;
+}
+
+sub xgt {
+  carp "usage: xgt(\\&sorter,$a,$b)" unless @_ and ref($_[0]);
+  #AKA: xcmp(@_) == -1;
+  return  0 if $_[1] eq $_[2];  # We have to trap this early.
+  return  1 if $_[1] ne ( $_[0]->($_[1], $_[2]) )[0];
+   # If they were switched when sorted, then the original-first was
+   #  lexically GT than the original-second.
+  return  0;
+}
+
+# And then two easy ones:
+
+sub xle {
+  carp "usage: xle(\\&sorter,$a,$b)" unless @_ and ref($_[0]);
+  !xgt(@_);    #AKA: xcmp(@_) <   1;
+}
+
+sub xge {
+  carp "usage: xge(\\&sorter,$a,$b)" unless @_ and ref($_[0]);
+  !xlt(@_);    #AKA: xcmp(@_) >  -1;
+}
+
 ###########################################################################
 1;
 
@@ -460,7 +513,7 @@ function, and try modifying that function on your own.
 
 =head1 DESCRIPTION
 
-This module provides two functions, C<Sort::ArbBiLex::maker> and
+This module provides two main functions, C<Sort::ArbBiLex::maker> and
 C<Sort::ArbBiLex::source_maker>, and it also presents an interface
 that accepts parameters in the C<use Sort::ArbBiLex ( ... )>
 statement.
@@ -668,6 +721,89 @@ whitespace as equal.  Au contraire!  They're glyphs, just like any
 others, so sorting "for sure" and "for  sure" (two spaces) is
 totally analogous to sorting "ika" and "ikka".
 
+=head1 COMPARATOR FUNCTIONS
+
+For most purposes, a sorter function generated by Sort::ArbBiLex is
+all you need.  However, in some cases, you need the equivalent of a
+language-specific C<cmp> function.  For example, consider this
+construct:
+
+  @records = sort { $a->{'headword'} cmp $b->{'headword'} }
+                  @records;
+
+If you think want a language-specific C<cmp>, you can work around it
+by start with a ArbBiLex-made function called C<my_sort>, and having:
+
+  {
+    my %hw2rec; # temporary mapping from headwords to records
+    for(@records) { push @{$hw2rec{ $_->{'headword'} }}, $_ }
+    @records = map @{$hw2rec{$_}}, my_sort( keys %hw2rec );
+  }
+
+and that's fine, and probably more time-efficient than what I'm about
+to suggest.
+
+If you really insist on having functions that act like C<cmp> and the
+other Perl comparator functions (C<lt>, C<gt>, C<le>, C<ge>), you can
+use these meta-functions:
+
+  Sort::ArbBiLex::xcmp(\&my_sort, $a,$b)
+  Sort::ArbBiLex::xlt( \&my_sort, $a,$b)
+  Sort::ArbBiLex::xgt( \&my_sort, $a,$b)
+  Sort::ArbBiLex::xle( \&my_sort, $a,$b)
+  Sort::ArbBiLex::xge( \&my_sort, $a,$b)
+
+Incidentally, these all work by seeing what the C<my_sort> function
+(i.e., whatever you pass a reference to) does when asked to sort the
+two values you pass.  Ideally, language-specific comparators would
+instead have been implemented by generating new comparator functions
+based on sort-order declarations, i.e., the same way we get sort
+functions.  However, comparators are needed rarely enough, and in
+already inefficient enough settings, that I sacrifice comparators'
+efficiency for the sake of the clarity of the module.
+
+So a concise way to say this
+
+  @records = sort { $a->{'headword'} cmp $b->{'headword'} }
+                  @records;
+
+with language-specific sorting is:
+
+  @records = sort { 
+    Sort::ArbBiLex::xcmp(\&my_sort,
+                         $a->{'headword'}, $b->{'headword'})
+  } @records;
+
+If you'd be doing that a lot, you can even wrap that comparator in a
+new function:
+
+ sub my_cmp { Sort::ArbBiLex::xcmp(\&my_sort, @_) };
+
+The full repertory of these would be:
+
+ use Sort::ArbBiLex ('my_sort' => ...whatever...);
+ sub my_cmp { Sort::ArbBiLex::xcmp(\&my_sort, @_) };
+ sub my_lt  { Sort::ArbBiLex::xlt( \&my_sort, @_) };
+ sub my_gt  { Sort::ArbBiLex::xgt( \&my_sort, @_) };
+ sub my_le  { Sort::ArbBiLex::xle( \&my_sort, @_) };
+ sub my_ge  { Sort::ArbBiLex::xge( \&my_sort, @_) };
+
+Using these functions makes for very readable code, like so:
+
+  @records = sort { my_cmp($a->{'headword'}, $b->{'headword'})}
+                  @records;
+
+but this will be almost definitely be much less time-efficient, for
+large lists, than the workaround mentioned at the top of this section.
+Benchmark both ways if you really need to know which is faster for
+your data set.  If neither are as fast as you really need them to be,
+then use C<sort_maker> to generate source code for a given
+declaration, and fiddle with that code to make comparators; the result
+should be much faster, even though it may take you a bit of doing.  As
+always, though, don't try optimizing unless you're sure you need to
+(and don't be surprised if it doesn't have as great an effect as you
+hoped for).
+
 =head1 GUTS
 
 * ArbBiLex stands for "arbitrary bi-level lexicographic".
@@ -721,10 +857,14 @@ name on it, like "MySorts::nornish").
 
 =head1 COPYRIGHT
 
-Copyright 1999, 2000, Sean M. Burke C<sburke@cpan.org>, all rights
+Copyright 1999-2001, Sean M. Burke C<sburke@cpan.org>, all rights
 reserved.  This program is free software; you can redistribute it
 and/or modify it under the same terms as Perl itself.
 
+This program is distributed in the hope that it will be useful, but
+without any warranty; without even the implied warranty of
+merchantability or fitness for a particular purpose.
+
 =head1 AUTHOR
 
 Sean M. Burke C<sburke@cpan.org>
